@@ -18,7 +18,7 @@
 #
 #----------------------------------------------------------------------------
 #
-# $Id: Stash.pm,v 2.20 2001/06/15 14:30:56 abw Exp $
+# $Id: Stash.pm,v 2.25 2001/06/29 13:09:00 abw Exp $
 #
 #============================================================================
 
@@ -29,7 +29,7 @@ require 5.004;
 use strict;
 use vars qw( $VERSION $DEBUG $ROOT_OPS $SCALAR_OPS $HASH_OPS $LIST_OPS );
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.20 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.25 $ =~ /(\d+)\.(\d+)/);
 
 
 #========================================================================
@@ -56,60 +56,61 @@ $SCALAR_OPS = {
     'length'  => sub { length $_[0] },
     'defined' => sub { return 1 },
     'repeat'  => sub { 
-	my ($str, $count) = @_;
-	$str = '' unless defined $str;  $count ||= 1;
-	return $str x $count;
+        my ($str, $count) = @_;
+        $str = '' unless defined $str;  
+        $count ||= 1;
+        return $str x $count;
     },
     'search'  => sub { 
-	my ($str, $pattern) = @_;
-	return $str unless defined $str and $pattern;
-	return $str =~ /$pattern/;
+        my ($str, $pattern) = @_;
+        return $str unless defined $str and defined $pattern;
+        return $str =~ /$pattern/;
     },
     'replace'  => sub { 
-	my ($str, $search, $replace) = @_;
-	$replace = '' unless defined $replace;
-	return $str unless defined $str and $search;
-	$str =~ s/$search/$replace/g;
-#	print STDERR "s [ $search ] [ $replace ] g\n";
-#	eval "\$str =~ s$search$replaceg";
-	return $str;
+        my ($str, $search, $replace) = @_;
+        $replace = '' unless defined $replace;
+        return $str unless defined $str and defined $search;
+        $str =~ s/$search/$replace/g;
+#       print STDERR "s [ $search ] [ $replace ] g\n";
+#       eval "\$str =~ s$search$replaceg";
+        return $str;
     },
     'split'   => sub { 
-	my ($str, $split, @args) = @_;
-	$str = '' unless defined $str;
-	return [ defined $split ? split($split, $str, @args)
-				: split(' ', $str, @args) ];
+        my ($str, $split, @args) = @_;
+        $str = '' unless defined $str;
+        return [ defined $split ? split($split, $str, @args)
+                                : split(' ', $str, @args) ];
     },
     defined $SCALAR_OPS ? %$SCALAR_OPS : (),
 };
 
 $HASH_OPS = {
     'item'   => sub { my ($hash, $item) = @_; 
-		      $item = '' unless defined $item;
-		      $hash->{ $item };
-		  },
+                      $item = '' unless defined $item;
+                      $hash->{ $item };
+                  },
     'hash'   => sub { $_[0] },
     'keys'   => sub { [ keys   %{ $_[0] } ] },
     'values' => sub { [ values %{ $_[0] } ] },
     'each'   => sub { [        %{ $_[0] } ] },
     'list'   => sub { my ($hash, $what) = @_;  $what ||= '';
-		      return ($what eq 'keys')   ? [   keys %$hash ]
-			   : ($what eq 'values') ? [ values %$hash ]
-			   : ($what eq 'each')   ? [ values %$hash ]
-			   : [ map { { key => $_ , value => $hash->{ $_ } } }
-			       keys %$hash ];
-		},
+                      return ($what eq 'keys')   ? [   keys %$hash ]
+                           : ($what eq 'values') ? [ values %$hash ]
+                           : ($what eq 'each')   ? [ values %$hash ]
+                           : [ map { { key => $_ , value => $hash->{ $_ } } }
+                               keys %$hash ];
+                },
     'import' => sub { my ($hash, $imp) = @_;
-		      $imp = {} unless ref $imp eq 'HASH';
-		      @$hash{ keys %$imp } = values %$imp;
-		      return '';
-		  },
+                      $imp = {} unless ref $imp eq 'HASH';
+                      @$hash{ keys %$imp } = values %$imp;
+                      return '';
+                  },
     'sort'    => sub {
-	my ($hash) = @_;
+        my ($hash) = @_;
         [ sort { lc $hash->{$a} cmp lc $hash->{$b} } (keys %$hash) ];
     },
     'nsort'    => sub {
-	my ($hash) = @_;
+        my ($hash) = @_;
         [ sort { $hash->{$a} <=> $hash->{$b} } (keys %$hash) ];
     },
     defined $HASH_OPS ? %$HASH_OPS : (),
@@ -119,7 +120,7 @@ $LIST_OPS = {
     'item'    => sub { $_[0]->[ $_[1] || 0 ] },
     'list'    => sub { $_[0] },
     'hash'    => sub { my $list = shift; my $n = 0; 
-		       return { map { ($n++, $_) } @$list }; },
+                       return { map { ($n++, $_) } @$list }; },
     'push'    => sub { my $list = shift; push(@$list, shift); return '' },
     'pop'     => sub { my $list = shift; pop(@$list) },
     'unshift' => sub { my $list = shift; unshift(@$list, shift); return '' },
@@ -130,35 +131,35 @@ $LIST_OPS = {
     'last'    => sub { my $list = shift; $list->[$#$list] },
     'reverse' => sub { my $list = shift; [ reverse @$list ] },
     'join'    => sub { 
-	    my ($list, $joint) = @_; 
-	    join(defined $joint ? $joint : ' ', 
-		 map { defined $_ ? $_ : '' } @$list) 
+            my ($list, $joint) = @_; 
+            join(defined $joint ? $joint : ' ', 
+                 map { defined $_ ? $_ : '' } @$list) 
     },
     'sort'    => sub {
-	my ($list, $field) = @_;
-	return $list unless $#$list;	    # no need to sort 1 item lists
-	return $field			    # Schwartzian Transform 
-	    ?  map  { $_->[0] }		    # for case insensitivity
-	       sort { $a->[1] cmp $b->[1] }
-	       map  { [ $_, lc $_->{ $field } ] } 
-	       @$list 
-	    :  map  { $_->[0] }
-	       sort { $a->[1] cmp $b->[1] }
-	       map  { [ $_, lc $_ ] } 
-	       @$list
+        my ($list, $field) = @_;
+        return $list unless $#$list;        # no need to sort 1 item lists
+        return $field                       # Schwartzian Transform 
+            ?  map  { $_->[0] }             # for case insensitivity
+               sort { $a->[1] cmp $b->[1] }
+               map  { [ $_, lc $_->{ $field } ] } 
+               @$list 
+            :  map  { $_->[0] }
+               sort { $a->[1] cmp $b->[1] }
+               map  { [ $_, lc $_ ] } 
+               @$list
    },
    'nsort'    => sub {
-	my ($list, $field) = @_;
-	return $list unless $#$list;	    # no need to sort 1 item lists
-	return $field			    # Schwartzian Transform 
-	    ?  map  { $_->[0] }		    # for case insensitivity
-	       sort { $a->[1] <=> $b->[1] }
-	       map  { [ $_, lc $_->{ $field } ] } 
-	       @$list 
-	    :  map  { $_->[0] }
-	       sort { $a->[1] <=> $b->[1] }
-	       map  { [ $_, lc $_ ] } 
-	       @$list
+        my ($list, $field) = @_;
+        return $list unless $#$list;        # no need to sort 1 item lists
+        return $field                       # Schwartzian Transform 
+            ?  map  { $_->[0] }             # for case insensitivity
+               sort { $a->[1] <=> $b->[1] }
+               map  { [ $_, lc $_->{ $field } ] } 
+               @$list 
+            :  map  { $_->[0] }
+               sort { $a->[1] <=> $b->[1] }
+               map  { [ $_, lc $_ ] } 
+               @$list
     },
     defined $LIST_OPS ? %$LIST_OPS : (),
 };
@@ -476,6 +477,10 @@ sub _dotop {
 	elsif ($value = $HASH_OPS->{ $item }) {
 	    @result = &$value($root, @$args);		    ## @result
 	}
+       elsif ( ref $item eq 'ARRAY' ) {
+             # hash slice
+             return [@$root{@$item}];                       ## RETURN
+       }
     }
     elsif ($rootref eq 'ARRAY') {
 
@@ -491,6 +496,10 @@ sub _dotop {
 	    return $value unless ref $value eq 'CODE';	    ## RETURN
 	    @result = &$value(@$args);			    ## @result
 	}
+        elsif ( ref $item eq 'ARRAY' ) {
+            # array slice
+            return [@$root[@$item]];                        ## RETURN
+        }
     }
 
     # NOTE: we do the can-can because UNIVSERAL::isa($something, 'UNIVERSAL')
@@ -506,6 +515,13 @@ sub _dotop {
 	eval { @result = $root->$item(@$args); };	    
 
 	if ($@) {
+	    # temporary hack - required to propogate errors thrown
+	    # by views; if $@ is a ref (e.g. Template::Exception
+	    # object then we assume it's a real error that needs
+	    # real throwing
+
+	    die $@ if ref($@) || ($@ !~ /Can't locate object method/);
+
 	    # failed to call object method, so try some fallbacks
 	    if (UNIVERSAL::isa($root, 'HASH')
 		&& defined($value = $root->{ $item })) {
@@ -516,7 +532,7 @@ sub _dotop {
 		   && ($value = $LIST_OPS->{ $item })) {
 		@result = &$value($root, @$args);
 	    }
-	    else {
+	    elsif ($self->{ _DEBUG }) {
 		@result = (undef, $@);
 	    }
 	}
@@ -794,8 +810,8 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.19, distributed as part of the
-Template Toolkit version 2.03, released on 15 June 2001.
+2.25, distributed as part of the
+Template Toolkit version 2.04, released on 29 June 2001.
 
 =head1 COPYRIGHT
 
